/* FILE: fileio.h                 -*-Mode: c++-*-
 *
 * File input/output routines
 *
 */

#ifndef FILEIO_H
#define FILEIO_H

#include <stdio.h>

#include "misc.h"
#include "template.h"
#include "vector.h"

/* End includes */     /* Optional directive to build.tcl */

class VioFile {
private:
  static const ClassDoc class_doc;
  BOOL swap_input_bytes; // If true, swap bytes on input
  FILE *fptr;
  INT4m width,height,depth;
  INT4m ReadHeader(); // Assumes fptr is correctly positioned
public:
  VioFile(void) { fptr=(FILE *)NULL; width=height=depth=0; }
  ~VioFile(void) { CloseFile(); }

  INT4m OpenFileForInput(const char *filename);

  INT4m SetFilePtr(FILE *newfptr) { 
    CloseFile();  fptr=newfptr;
    return ReadHeader();
  }

  INT4m GetDimensions(INT4m &xwidth,INT4m &yheight,INT4m &zdepth) {
    xwidth=width; yheight=height; zdepth=depth;
    return width*height*depth;
  }

  INT4m FillArray(GridVec3f &grid);
  // Reads magnetization vector array; Assumes fptr is correctly positioned

  void CloseFile(void) { 
    if(fptr!=(FILE *)NULL) fclose(fptr);
    fptr=(FILE *)NULL;
  }
};

#endif // FILEIO_H
