/* FILE: mmdisp.cc                 -*-Mode: c++-*-
 *
 * Mainline for C++/Tcl/Tk 2D vector field display program
 *
 */

#include "machine.h"	/* SYSTEM_TYPE, ... */

// Eventually get build.tcl to find this header file.
#include "../oommf/oommf.h"	/* Oommf_Init */

#include "tkcmds.h"	/* Mmdisp_Init, TkCommandsPreInit, ... */

#include <tcl.h>	/* TCL_OK, ... */
#include <tk.h>		/* Tk_Main */
#include <stdio.h>	/* freopen */
#include <stdlib.h>	/* strcmp (?), ... */

/* End includes */     // This is an optional directive to build.tcl,
                       // that excludes the remainder of the file from
                       // '#include ".*"' dependency building

int Tcl_AppInit(Tcl_Interp *interp)
{
    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
#if (TCL_MAJOR_VERSION >= 7) && (TCL_MINOR_VERSION >= 5)
// Disable this until it is known to be useful.
// We think it's causing link trouble.
//    Tcl_StaticPackage(interp, "Tk", Tk_Init, (Tcl_PackageInitProc *) NULL);
#endif
    if (Oommf_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    if (Mmdisp_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }
    return TCL_OK;
}

void Usage(void)
{
  fprintf(stderr,"Usage: mmdisp [--] [filename]\n");
  exit(1);
}

void CheckParams(int argc,char **argv, char* &filename)
{
  if(argc<2)                             { filename=NULL;    return; }
  if(argc==2 && argv[1][0]!='-')         { filename=argv[1]; return; }
  if(argc==3 && strcmp(argv[1],"--")==0) { filename=argv[2]; return; }
  Usage();
}

#if (SYSTEM_TYPE==WINDOWS)
int APIENTRY WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,
		     LPSTR lpszCmdLine,int nCmdShow)
{
  int argc;
  char *argv[1];
  argv[0]=new char[strlen("mmdisp")+1];
  strcpy(argv[0],"mmdisp");

  TkCommandsPreInit(argv[0],lpszCmdLine);

  argc=1; /* Tk_Main will try to input command line arg's as input files */
  Tk_Main(argc,argv,Tcl_AppInit);

  delete argv[0];  // Note: Tk_Main doesn't return, so argv doesn't
                   //  actually get deleted.  I guess the way to do
                   //  this is to pass the pointer to TkCommandsCleanup
                   //  for deletion, but this is probably not worth the
                   //  trouble for at most a couple dozen bytes.
  return 0;
}
#elif (SYSTEM_TYPE == UNIX)
int main(int argc,char **argv)
{
  char *filename;
  CheckParams(argc,argv,filename);

#if DEBUGLVL<1
  if(fork()==0) {
    // Child process
    freopen("/dev/null","r",stdin);
    freopen("/dev/null","r",stdout);
#endif // DEBUGLVL<1
    TkCommandsPreInit(argv[0],filename);
    argc=1; /* Tk_Main will try to input command line arg's as input files */
    Tk_Main(argc,argv,Tcl_AppInit);
#if DEBUGLVL<1
  }
#endif // DEBUGLVL<1

  return 0;
}
#else
#error "Unknown SYSTEM_TYPE"
#endif // SYSTEM_TYPE == WINDOWS
