#include <stdio.h>
#include <string.h>

/* Debugging routine */
void DumpBytes(buf,len)
unsigned char *buf;
int len;
{
  int i;
  for(i=0;i<len;i++) printf("%02X ",buf[i]);
  printf("\n");
}

int main()
{
  /* Work length info */
  printf("Type        char is %2d bytes long\n\n",sizeof(char));
  printf("Type       short is %2d bytes long\n",sizeof(short));
  printf("Type         int is %2d bytes long\n",sizeof(int));
  printf("Type        long is %2d bytes long\n",sizeof(long));
#ifdef EXTRALONG
  printf("Type   long long is %2d bytes long\n",sizeof(long long));
#endif /* EXTRALONG */

  printf("\n");
  printf("Type       float is %2d bytes long\n",sizeof(float));
  printf("Type      double is %2d bytes long\n",sizeof(double));
#ifdef EXTRALONG
  printf("Type long double is %2d bytes long\n",sizeof(long double));
#endif /* EXTRALONG */

  printf("\n");
  printf("Type      void * is %2d bytes long\n",sizeof(void *));

#ifdef COMPLEX
  printf("\n");
  printf("Type __complex__    int is %2d bytes long\n",
	 sizeof(__complex__ int));
  printf("Type __complex__ double is %2d bytes long\n",
	 sizeof(__complex__ double));
#endif /* COMPLEX */

  /* Byte order info */
  printf("\n"); fflush(stdout);
  if(sizeof(double)!=8) {
    fprintf(stderr,"Can't test byte order; sizeof(double)!=8\n");
  }
  else {
    union {
      double x;
      unsigned char c[8];
    } bytetest;
    double xpattern=1.234;
    unsigned char lsbpattern[9],msbpattern[9];
    strncpy(lsbpattern,"\130\071\264\310\166\276\363\077",
	    sizeof(lsbpattern));
    strncpy(msbpattern,"\077\363\276\166\310\264\071\130",
	    sizeof(msbpattern));

    /* The bit pattern for 1.234 on a little endian machine (e.g.,
     * Intel's x86 series and Digital's AXP) should be
     * 58 39 B4 C8 76 BE F3 3F, while on a big endian machine it is
     * 3F F3 BE 76 C8 B4 39 58.  Examples of big endian machines
     * include ones using the MIPS R4000 and R8000 series chips,
     * for example SGI's.  Note: At least some versions of the Sun
     * 'cc' compiler apparently don't grok '\xXX' hex notation.  The
     * above octal constants are equivalent to the aforementioned
     * hex strings.
     */
    bytetest.x=xpattern;

#ifdef DEBUG    
    printf("sizeof(lsbpattern)=%d\n",sizeof(lsbpattern));
    printf("lsb pattern="); DumpBytes(lsbpattern,8);
    printf("msb pattern="); DumpBytes(msbpattern,8);
    printf("  x pattern="); DumpBytes(bytetest.c,8);
#endif /* DEBUG */

    if(strncmp(bytetest.c,lsbpattern,8)==0) {
      printf("Floating point format is IEEE in LSB order\n");
    }
    else if(strncmp(bytetest.c,msbpattern,8)==0) {
      printf("Floating point format is IEEE in MSB order\n");
    }
    else {
      printf("Floating point format is either unknown or uses an irregular byte order\n");
    }
  }
  return 0;
}
