/* FILE: viotoomf.cc                 -*-Mode: c++-*-
 *
 * Program to convert from vio to omf vector field file formats.
 *
 */

#include <stdio.h>
#include "machine.h"
#include "misc.h"
#include "template.h"
#include "vector.h"
#include "vecfile.h"

void Usage(void)
{
  fprintf(stderr,"Usage: viotoomf [input.vio |<input.vio] >output.omf\n");
  exit(1);
}

int main(int argc,char **argv)
{
  if(argc>2
     || (argc==2 && argv[1][0]=='-' && argv[1][1]=='h' && argv[1][2]=='\0'))
    Usage();

  FILE *infile;
  char *inname,*omfname;

  if(argc==2) {
    inname=argv[1];
    infile=fopen(inname,"rb");
    omfname=new char[strlen(inname)+1];
    strcpy(omfname,inname);
    char *cptr=strstr(omfname,".vio");
    if(cptr!=NULL) strcpy(cptr,".omf");
  }
  else {
    infile=stdin;
    inname="stdin";
    omfname=new char[strlen("unknown.omf")+1];
    strcpy(omfname,"unknown.omf");
  }

  if(infile==NULL) {
    fprintf(stderr,"ERROR: Unable to open %s for input\n",inname);
    Usage();
  }

  // Open and read vio file
  VioFileInput vfi(inname,infile);
  GridVec3f *grid=vfi.NewRectGrid();

  INT4m xdim,ydim,zdim;
  grid->GetSize(xdim,ydim,zdim);

  if(zdim!=1) {
    fprintf(stderr,"Illegal z dimension in input file: %d (should be 1)\n",
	    zdim);
    exit(1);
  }

  // Write omf file
  fprintf(stdout,"# OOMMF: rectangular mesh v0.99\n");
  fprintf(stdout,"# Segment count: 1\n");
  fprintf(stdout,"# Begin: Segment\n");
  fprintf(stdout,"# Begin: Header\n");
  fprintf(stdout,"# Title: %s\n",omfname);
  fprintf(stdout,"# Desc: Omf file created from vio file by viotoomf\n");
  fprintf(stdout,"# meshunit: unknown\n");
  fprintf(stdout,"# xbase: 0\n# ybase: 0\n# zbase: 0\n");
  fprintf(stdout,"# xstepsize: 1\n# ystepsize: 1\n# zstepsize: 1\n");
  fprintf(stdout,"# xnodes: %d\n",xdim);
  fprintf(stdout,"# ynodes: %d\n",ydim);
  fprintf(stdout,"# znodes: 1\n");
  fprintf(stdout,"# xmin: -0.5\n# ymin: -0.5\n# zmin: -0.5\n");
  fprintf(stdout,"# xmax: %.1f\n# ymax: %.1f\n# zmax:  0.5\n",
	  double(xdim)-0.5,double(ydim)-0.5);
  fprintf(stdout,"# valueunit: Ms\n");
  fprintf(stdout,"# valuemultiplier: 1\n");
  fprintf(stdout,"# ValueRangeMaxMag:  1.\n");
  fprintf(stdout,"# ValueRangeMinMag:  1e-8\n");
  fprintf(stdout,"# End: Header\n");

  // Print grid data
  fprintf(stdout,"# Begin: data text\n");
  INT4m i,j,k;
  Vec3f v;
  for(k=0;k<zdim;k++) for(j=0;j<ydim;j++) for(i=0;i<xdim;i++) {
    v=grid->GridVec(i,j,k);
    fprintf(stdout,"% -14.8g % -14.8g % -14.8g\n",v.x,v.y,v.z);
  }
  fprintf(stdout,"# End: data text\n");
  fprintf(stdout,"# End: segment\n");

  // Clean up
  fflush(stdout);
  delete grid;
  delete[] omfname;
  return 0;
}
