/* FILE: machine.h                 -*-Mode: c++-*-
 *
 * Machine specific #define's and typedef's.
 *
 */

#ifndef MACHINE_H
#define MACHINE_H

/* End includes */     /* Optional directive to build.tcl */

// Known machine types
#define LINUX_AXP      1
#define LINUX_INTEL    2
#define SGI_64         3
#define SUN_SPARC      4
#define WIN_INTEL      5

#ifndef MACHINE_TYPE
#if defined(__alpha__) && defined(__linux__)
#define MACHINE_TYPE LINUX_AXP
#elif defined(__i386__)  && defined(__linux__)
#define MACHINE_TYPE LINUX_INTEL
#elif defined(__sparc__) && defined(__sun__)
#define MACHINE_TYPE SUN_SPARC
#elif defined(__sparc) && defined(__sun)
#define MACHINE_TYPE SUN_SPARC
#elif defined(__mips) && defined(__sgi) && (__mips == 4)
#define MACHINE_TYPE SGI_64
#elif defined(__mips) && defined(__sgi)
#define MACHINE_TYPE SGI_32
#else
#error "Unable to identify Machine/OS"
#endif

#endif // MACHINE_TYPE

// Known system types
#define UNIX 1
#define WINDOWS 2

#if (MACHINE_TYPE == WIN_INTEL)
#define SYSTEM_TYPE WINDOWS
#else
#define SYSTEM_TYPE UNIX    // Default to unix
#endif

#if (MACHINE_TYPE == LINUX_INTEL) || (MACHINE_TYPE == SUN_SPARC) \
 || (MACHINE_TYPE == SGI_32)
// Standard 32-bit machines
typedef  float              REAL4;
typedef  double             REAL8;
typedef  short              INT2;
typedef  int                INT4;
typedef  unsigned short     UINT2;
typedef  unsigned int       UINT4;
#ifdef __gcc__
typedef  long long          INT8; 
typedef  unsigned long long UINT8;
#endif // __gcc__
typedef  int                BOOL;
typedef  char               CHAR;
typedef  signed char        SCHAR;
typedef  unsigned char      UCHAR;
typedef  float              REAL4m;
typedef  double             REAL8m;
typedef  int                INT2m;
typedef  int                INT4m;
typedef  unsigned int       UINT2m;
typedef  unsigned int       UINT4m;
#ifdef __gcc__
typedef  long long          INT8m; 
typedef  unsigned long long UINT8m;
#endif // __gcc__
#elif (MACHINE_TYPE == LINUX_AXP) || (MACHINE_TYPE == SGI_64)
// Standard 64-bit machines
typedef  float              REAL4;
typedef  double             REAL8;
typedef  short              INT2;
typedef  int                INT4;
typedef  long               INT8;
typedef  unsigned short     UINT2;
typedef  unsigned int       UINT4;
typedef  unsigned long      UINT8;
typedef  int                BOOL;
typedef  char               CHAR;
typedef  signed char        SCHAR;
typedef  unsigned char      UCHAR;
typedef  float              REAL4m;
typedef  double             REAL8m;
typedef  int                INT2m;
typedef  int                INT4m;
typedef  long               INT8m;
typedef  unsigned int       UINT2m;
typedef  unsigned int       UINT4m;
typedef  unsigned long      UINT8m;
#elif (MACHINE_TYPE == WIN_INTEL)
// Windows/Intel
typedef  float              REAL4;
typedef  double             REAL8;
typedef  short              INT2;
typedef  int                INT4;
typedef  unsigned short     UINT2;
typedef  unsigned int       UINT4;
typedef  int                BOOL;
typedef  char               CHAR;
typedef  signed char        SCHAR;
typedef  unsigned char      UCHAR;
typedef  float              REAL4m;
typedef  double             REAL8m;
typedef  int                INT2m;
typedef  int                INT4m;
typedef  unsigned int       UINT2m;
typedef  unsigned int       UINT4m;
#else
#error "Unknown MACHINE_TYPE"
#endif

#define MEMBLOCKSIZE 4096 // Natural system memory blocksize.  Shouldn't
               // affect program correctness, but may affect performance.

// Include files that may be system dependent.
#include <stdlib.h>
#include <string.h>

#if (SYSTEM_TYPE == WINDOWS)
#define rint(x) ((x)>=0 ? double(int(x+0.5)) : double(int(x-0.5)))
// A few definitions taken from windows' include files
#define WINAPI __stdcall
#define APIENTRY WINAPI
#define __RPC_FAR
typedef void __RPC_FAR *HINSTANCE;
typedef char *LPSTR;
#endif
#if (SYSTEM_TYPE == UNIX)
#include <unistd.h>
#endif

// Directory path separator; Unix uses '/', DOS uses '\'.
// Rumor has it that the Mac uses ':'.
#define DIRSEPCHAR '/'
#define DIRSEPSTR  "/"

// Separator between directory elements in environment "PATH" variable
#define PATHSPLITSTR ":"

#endif // MACHINE_H
